@@ -17,22 +17,27 @@ use vars qw{%config %links %oldlinks %pagemtime %pagectime %pagecase
 	    %forcerebuild %loaded_plugins};
 
 use Exporter q{import};
-our @EXPORT = qw(hook debug error template htmlpage add_depends pagespec_match
-                 pagespec_match_list bestlink htmllink readfile writefile
-		 pagetype srcfile pagename displaytime will_render gettext urlto
-		 targetpage add_underlay pagetitle titlepage linkpage
-		 newpagefile inject add_link
+our @EXPORT = qw(hook debug error template htmlpage deptype
+                 add_depends pagespec_match pagespec_match_list bestlink
+		 htmllink readfile writefile pagetype srcfile pagename
+		 displaytime will_render gettext ngettext urlto targetpage
+		 add_underlay pagetitle titlepage linkpage newpagefile
+		 inject add_link
                  %config %links %pagestate %wikistate %renderedfiles
                  %pagesources %destsources);
 our $VERSION = 3.00; # plugin interface version, next is ikiwiki version
 our $version='unknown'; # VERSION_AUTOREPLACE done by Makefile, DNE
 our $installdir='/usr'; # INSTALLDIR_AUTOREPLACE done by Makefile, DNE
 
+# Page dependency types.
+our $DEPEND_CONTENT=1;
+our $DEPEND_PRESENCE=2;
+our $DEPEND_LINKS=4;
+
 # Optimisation.
 use Memoize;
 memoize("abs2rel");
 memoize("pagespec_translate");
-memoize("file_pruned");
 memoize("template_file");
 
 sub getsetup () {
@@ -404,6 +409,13 @@ sub getsetup () {
 		safe => 0,
 		rebuild => 0,
 	},
+	clean => {
+		type => "internal",
+		default => 0,
+		description => "running in clean mode",
+		safe => 0,
+		rebuild => 0,
+	},
 	refresh => {
 		type => "internal",
 		default => 0,
@@ -544,12 +556,12 @@ sub loadplugins () {
 	}
 	
 	if ($config{rcs}) {
-		if (exists $IkiWiki::hooks{rcs}) {
+		if (exists $hooks{rcs}) {
 			error(gettext("cannot use multiple rcs plugins"));
 		}
 		loadplugin($config{rcs});
 	}
-	if (! exists $IkiWiki::hooks{rcs}) {
+	if (! exists $hooks{rcs}) {
 		loadplugin("norcs");
 	}
 
@@ -876,7 +888,7 @@ sub bestlink ($$) {
 		$l.="/" if length $l;
 		$l.=$link;
 
-		if (exists $links{$l}) {
+		if (exists $pagesources{$l}) {
 			return $l;
 		}
 		elsif (exists $pagecase{lc $l}) {
@@ -886,7 +898,7 @@ sub bestlink ($$) {
 
 	if (length $config{userdir}) {
 		my $l = "$config{userdir}/".lc($link);
-		if (exists $links{$l}) {
+		if (exists $pagesources{$l}) {
 			return $l;
 		}
 		elsif (exists $pagecase{lc $l}) {
@@ -936,7 +948,12 @@ sub linkpage ($) {
 sub cgiurl (@) {
 	my %params=@_;
 
-	return $config{cgiurl}."?".
+	my $cgiurl=$config{cgiurl};
+	if (exists $params{cgiurl}) {
+		$cgiurl=$params{cgiurl};
+		delete $params{cgiurl};
+	}
+	return $cgiurl."?".
 		join("&amp;", map $_."=".uri_escape_utf8($params{$_}), keys %params);
 }
 
@@ -1076,16 +1093,20 @@ sub htmllink ($$$;@) {
 	}
 
 	my @attrs;
-	if (defined $opts{rel}) {
-		push @attrs, ' rel="'.$opts{rel}.'"';
-	}
-	if (defined $opts{class}) {
-		push @attrs, ' class="'.$opts{class}.'"';
+	foreach my $attr (qw{rel class title}) {
+		if (defined $opts{$attr}) {
+			push @attrs, " $attr=\"$opts{$attr}\"";
+		}
 	}
 
 	return "<a href=\"$bestlink\"@attrs>$linktext</a>";
 }
 
+sub userpage ($) {
+	my $user=shift;
+	return length $config{userdir} ? "$config{userdir}/$user" : $user;
+}
+
 sub openiduser ($) {
 	my $user=shift;
 
@@ -1094,11 +1115,10 @@ sub openiduser ($) {
 		my $display;
 
 		if (Net::OpenID::VerifiedIdentity->can("DisplayOfURL")) {
-			# this works in at least 2.x
 			$display = Net::OpenID::VerifiedIdentity::DisplayOfURL($user);
 		}
 		else {
-			# this only works in 1.x
+			# backcompat with old version
 			my $oid=Net::OpenID::VerifiedIdentity->new(identity => $user);
 			$display=$oid->display;
 		}
@@ -1121,23 +1141,6 @@ sub openiduser ($) {
 	return;
 }
 
-sub userlink ($) {
-	my $user=shift;
-
-	my $oiduser=eval { openiduser($user) };
-	if (defined $oiduser) {
-		return "<a href=\"$user\">$oiduser</a>";
-	}
-	else {
-		eval q{use CGI 'escapeHTML'};
-		error($@) if $@;
-
-		return htmllink("", "", escapeHTML(
-			length $config{userdir} ? $config{userdir}."/".$user : $user
-		), noimageinline => 1);
-	}
-}
-
 sub htmlize ($$$$) {
 	my $page=shift;
 	my $destpage=shift;
@@ -1224,7 +1227,7 @@ sub preprocess ($$$;$$) {
 				(?:
 					"""(.*?)"""	# 2: triple-quoted value
 				|
-					"([^"]+)"	# 3: single-quoted value
+					"([^"]*?)"	# 3: single-quoted value
 				|
 					(\S+)		# 4: unquoted value
 				)
@@ -1310,7 +1313,7 @@ sub preprocess ($$$;$$) {
 					(?:
 						""".*?"""	# triple-quoted value
 						|
-						"[^"]+"		# single-quoted value
+						"[^"]*?"	# single-quoted value
 						|
 						[^"\s\]]+	# unquoted value
 					)
@@ -1333,7 +1336,7 @@ sub preprocess ($$$;$$) {
 					(?:
 						""".*?"""	# triple-quoted value
 						|
-						"[^"]+"		# single-quoted value
+						"[^"]*?"	# single-quoted value
 						|
 						[^"\s\]]+	# unquoted value
 					)
@@ -1403,7 +1406,7 @@ sub check_content (@) {
 		my %old=map { $_ => 1 }
 		        split("\n", readfile(srcfile($pagesources{$params{page}})));
 		foreach my $line (split("\n", $params{content})) {
-			push @diff, $line if ! exists $old{$_};
+			push @diff, $line if ! exists $old{$line};
 		}
 		$params{diff}=join("\n", @diff);
 	}
@@ -1524,18 +1527,28 @@ sub loadindex () {
 				$links{$page}=$d->{links};
 				$oldlinks{$page}=[@{$d->{links}}];
 			}
-			if (exists $d->{depends_simple}) {
+			if (ref $d->{depends_simple} eq 'ARRAY') {
+				# old format
 				$depends_simple{$page}={
 					map { $_ => 1 } @{$d->{depends_simple}}
 				};
 			}
+			elsif (exists $d->{depends_simple}) {
+				$depends_simple{$page}=$d->{depends_simple};
+			}
 			if (exists $d->{dependslist}) {
+				# old format
 				$depends{$page}={
-					map { $_ => 1 } @{$d->{dependslist}}
+					map { $_ => $DEPEND_CONTENT }
+						@{$d->{dependslist}}
 				};
 			}
+			elsif (exists $d->{depends} && ! ref $d->{depends}) {
+				# old format
+				$depends{$page}={$d->{depends} => $DEPEND_CONTENT };
+			}
 			elsif (exists $d->{depends}) {
-				$depends{$page}={$d->{depends} => 1};
+				$depends{$page}=$d->{depends};
 			}
 			if (exists $d->{state}) {
 				$pagestate{$page}=$d->{state};
@@ -1581,11 +1594,11 @@ sub saveindex () {
 		};
 
 		if (exists $depends{$page}) {
-			$index{page}{$src}{dependslist} = [ keys %{$depends{$page}} ];
+			$index{page}{$src}{depends} = $depends{$page};
 		}
 
 		if (exists $depends_simple{$page}) {
-			$index{page}{$src}{depends_simple} = [ keys %{$depends_simple{$page}} ];
+			$index{page}{$src}{depends_simple} = $depends_simple{$page};
 		}
 
 		if (exists $pagestate{$page}) {
@@ -1753,57 +1766,99 @@ sub rcs_receive () {
 	$hooks{rcs}{rcs_receive}{call}->();
 }
 
-sub add_depends ($$) {
+sub add_depends ($$;$) {
 	my $page=shift;
 	my $pagespec=shift;
+	my $deptype=shift || $DEPEND_CONTENT;
 
+	# Is the pagespec a simple page name?
 	if ($pagespec =~ /$config{wiki_file_regexp}/ &&
-		$pagespec !~ /[\s*?()!]/) {
-		# a simple dependency, which can be matched by string eq
-		$depends_simple{$page}{lc $pagespec} = 1;
+	    $pagespec !~ /[\s*?()!]/) {
+		$depends_simple{$page}{lc $pagespec} |= $deptype;
 		return 1;
 	}
 
-	return unless pagespec_valid($pagespec);
+	# Add explicit dependencies for influences.
+	my $sub=pagespec_translate($pagespec);
+	return if $@;
+	foreach my $p (keys %pagesources) {
+		my $r=$sub->($p, location => $page);
+		my $i=$r->influences;
+		foreach my $k (keys %$i) {
+			$depends_simple{$page}{lc $k} |= $i->{$k};
+		}
+		last if $r->influences_static;
+	}
 
-	$depends{$page}{$pagespec} = 1;
+	$depends{$page}{$pagespec} |= $deptype;
 	return 1;
 }
 
-sub file_pruned ($$) {
-	require File::Spec;
-	my $file=File::Spec->canonpath(shift);
-	my $base=File::Spec->canonpath(shift);
-	$file =~ s#^\Q$base\E/+##;
+sub deptype (@) {
+	my $deptype=0;
+	foreach my $type (@_) {
+		if ($type eq 'presence') {
+			$deptype |= $DEPEND_PRESENCE;
+		}
+		elsif ($type eq 'links') { 
+			$deptype |= $DEPEND_LINKS;
+		}
+		elsif ($type eq 'content') {
+			$deptype |= $DEPEND_CONTENT;
+		}
+	}
+	return $deptype;
+}
+
+sub file_pruned ($;$) {
+	my $file=shift;
+	if (@_) {
+		require File::Spec;
+		$file=File::Spec->canonpath($file);
+		my $base=File::Spec->canonpath(shift);
+		return if $file eq $base;
+		$file =~ s#^\Q$base\E/+##;
+	}
 
 	my $regexp='('.join('|', @{$config{wiki_file_prune_regexps}}).')';
-	return $file =~ m/$regexp/ && $file ne $base;
+	return $file =~ m/$regexp/;
 }
 
 sub define_gettext () {
 	# If translation is needed, redefine the gettext function to do it.
 	# Otherwise, it becomes a quick no-op.
-	no warnings 'redefine';
+	my $gettext_obj;
+	my $getobj;
 	if ((exists $ENV{LANG} && length $ENV{LANG}) ||
 	    (exists $ENV{LC_ALL} && length $ENV{LC_ALL}) ||
 	    (exists $ENV{LC_MESSAGES} && length $ENV{LC_MESSAGES})) {
-	    	*gettext=sub {
-			my $gettext_obj=eval q{
+	    	$getobj=sub {
+			$gettext_obj=eval q{
 				use Locale::gettext q{textdomain};
 				Locale::gettext->domain('ikiwiki')
 			};
-
-			if ($gettext_obj) {
-				$gettext_obj->get(shift);
-			}
-			else {
-				return shift;
-			}
 		};
 	}
-	else {
-		*gettext=sub { return shift };
-	}
+
+	no warnings 'redefine';
+	*gettext=sub {
+		$getobj->() if $getobj;
+		if ($gettext_obj) {
+			$gettext_obj->get(shift);
+		}
+		else {
+			return shift;
+		}
+	};
+  	*ngettext=sub {
+		$getobj->() if $getobj;
+		if ($gettext_obj) {
+			$gettext_obj->nget(@_);
+		}
+		else {
+			return ($_[2] == 1 ? $_[0] : $_[1])
+		}
+	};
 }
 
 sub gettext {
@@ -1811,6 +1866,11 @@ sub gettext {
 	gettext(@_);
 }
 
+sub ngettext {
+	define_gettext();
+	ngettext(@_);
+}
+
 sub yesno ($) {
 	my $val=shift;
 
@@ -1870,13 +1930,13 @@ sub pagespec_translate ($) {
 			[^\s()]+	# any other text
 		)
 		\s*		# ignore whitespace
-	}igx) {
+	}gx) {
 		my $word=$1;
 		if (lc $word eq 'and') {
-			$code.=' &&';
+			$code.=' &';
 		}
 		elsif (lc $word eq 'or') {
-			$code.=' ||';
+			$code.=' |';
 		}
 		elsif ($word eq "(" || $word eq ")" || $word eq "!") {
 			$code.=' '.$word;
@@ -1922,27 +1982,87 @@ sub pagespec_match ($$;@) {
 }
 
 sub pagespec_match_list ($$;@) {
-	my $pages=shift;
-	my $spec=shift;
-	my @params=@_;
+	my $page=shift;
+	my $pagespec=shift;
+	my %params=@_;
 
-	my $sub=pagespec_translate($spec);
-	error "syntax error in pagespec \"$spec\""
-		if $@ || ! defined $sub;
-	
-	my @ret;
-	my $r;
-	foreach my $page (@$pages) {
-		$r=$sub->($page, @params);
-		push @ret, $page if $r;
+	# Backwards compatability with old calling convention.
+	if (ref $page) {
+		print STDERR "warning: a plugin (".caller().") is using pagespec_match_list in an obsolete way, and needs to be updated\n";
+		$params{list}=$page;
+		$page=$params{location}; # ugh!
 	}
 
-	if (! @ret && defined $r && $r->isa("IkiWiki::ErrorReason")) {
-		error(sprintf(gettext("cannot match pages: %s"), $r));
+	my $sub=pagespec_translate($pagespec);
+	error "syntax error in pagespec \"$pagespec\""
+		if $@ || ! defined $sub;
+
+	my @candidates;
+	if (exists $params{list}) {
+		@candidates=exists $params{filter}
+			? grep { ! $params{filter}->($_) } @{$params{list}}
+			: @{$params{list}};
 	}
 	else {
-		return @ret;
+		@candidates=exists $params{filter}
+			? grep { ! $params{filter}->($_) } keys %pagesources
+			: keys %pagesources;
 	}
+
+	if (defined $params{sort}) {
+		my $f;
+		if ($params{sort} eq 'title') {
+			$f=sub { pagetitle(basename($a)) cmp pagetitle(basename($b)) };
+		}
+		elsif ($params{sort} eq 'title_natural') {
+			eval q{use Sort::Naturally};
+			if ($@) {
+				error(gettext("Sort::Naturally needed for title_natural sort"));
+			}
+			$f=sub { Sort::Naturally::ncmp(pagetitle(basename($a)), pagetitle(basename($b))) };
+                }
+		elsif ($params{sort} eq 'mtime') {
+			$f=sub { $pagemtime{$b} <=> $pagemtime{$a} };
+		}
+		elsif ($params{sort} eq 'age') {
+			$f=sub { $pagectime{$b} <=> $pagectime{$a} };
+		}
+		else {
+			error sprintf(gettext("unknown sort type %s"), $params{sort});
+		}
+		@candidates = sort { &$f } @candidates;
+	}
+
+	@candidates=reverse(@candidates) if $params{reverse};
+	
+	$depends{$page}{$pagespec} |= ($params{deptype} || $DEPEND_CONTENT);
+	
+	# clear params, remainder is passed to pagespec
+	my $num=$params{num};
+	delete @params{qw{num deptype reverse sort filter list}};
+	
+	my @matches;
+	my $firstfail;
+	my $count=0;
+	my $accum=IkiWiki::SuccessReason->new();
+	foreach my $p (@candidates) {
+		my $r=$sub->($p, %params, location => $page);
+		error(sprintf(gettext("cannot match pages: %s"), $r))
+			if $r->isa("IkiWiki::ErrorReason");
+		$accum |= $r;
+		if ($r) {
+			push @matches, $p;
+			last if defined $num && ++$count == $num;
+		}
+	}
+
+	# Add simple dependencies for accumulated influences.
+	my $i=$accum->influences;
+	foreach my $k (keys %$i) {
+		$depends_simple{$page}{lc $k} |= $i->{$k};
+	}
+
+	return @matches;
 }
 
 sub pagespec_valid ($) {
@@ -1962,37 +2082,66 @@ sub glob2re ($) {
 package IkiWiki::FailReason;
 
 use overload (
-	'""'	=> sub { ${$_[0]} },
+	'""'	=> sub { $_[0][0] },
 	'0+'	=> sub { 0 },
 	'!'	=> sub { bless $_[0], 'IkiWiki::SuccessReason'},
+	'&'	=> sub { $_[0]->merge_influences($_[1], 1); $_[0] },
+	'|'	=> sub { $_[1]->merge_influences($_[0]); $_[1] },
+	fallback => 1,
+);
+
+our @ISA = 'IkiWiki::SuccessReason';
+
+package IkiWiki::SuccessReason;
+
+use overload (
+	'""'	=> sub { $_[0][0] },
+	'0+'	=> sub { 1 },
+	'!'	=> sub { bless $_[0], 'IkiWiki::FailReason'},
+	'&'	=> sub { $_[1]->merge_influences($_[0], 1); $_[1] },
+	'|'	=> sub { $_[0]->merge_influences($_[1]); $_[0] },
 	fallback => 1,
 );
 
 sub new {
 	my $class = shift;
 	my $value = shift;
-	return bless \$value, $class;
+	return bless [$value, {@_}], $class;
+}
+
+sub influences {
+	my $this=shift;
+	$this->[1]={@_} if @_;
+	my %i=%{$this->[1]};
+	delete $i{""};
+	return \%i;
+}
+
+sub influences_static {
+	return ! $_[0][1]->{""};
+}
+
+sub merge_influences {
+	my $this=shift;
+	my $other=shift;
+	my $anded=shift;
+
+	if (! $anded || (($this || %{$this->[1]}) &&
+	                ($other || %{$other->[1]}))) {
+		foreach my $influence (keys %{$other->[1]}) {
+			$this->[1]{$influence} |= $other->[1]{$influence};
+		}
+	}
+	else {
+		# influence blocker
+		$this->[1]={};
+	}
 }
 
 package IkiWiki::ErrorReason;
 
 our @ISA = 'IkiWiki::FailReason';
 
-package IkiWiki::SuccessReason;
-
-use overload (
-	'""'	=> sub { ${$_[0]} },
-	'0+'	=> sub { 1 },
-	'!'	=> sub { bless $_[0], 'IkiWiki::FailReason'},
-	fallback => 1,
-);
-
-sub new {
-	my $class = shift;
-	my $value = shift;
-	return bless \$value, $class;
-};
-
 package IkiWiki::PageSpec;
 
 sub derel ($$) {
@@ -2042,27 +2191,30 @@ sub match_link ($$;@) {
 	my $from=exists $params{location} ? $params{location} : '';
 
 	my $links = $IkiWiki::links{$page};
-	return IkiWiki::FailReason->new("$page has no links") unless $links && @{$links};
+	return IkiWiki::FailReason->new("$page has no links", "" => 1)
+		unless $links && @{$links};
 	my $bestlink = IkiWiki::bestlink($from, $link);
 	foreach my $p (@{$links}) {
 		if (length $bestlink) {
-			return IkiWiki::SuccessReason->new("$page links to $link")
+			return IkiWiki::SuccessReason->new("$page links to $link", $page => $IkiWiki::DEPEND_LINKS, "" => 1)
 				if $bestlink eq IkiWiki::bestlink($page, $p);
 		}
 		else {
-			return IkiWiki::SuccessReason->new("$page links to page $p matching $link")
+			return IkiWiki::SuccessReason->new("$page links to page $p matching $link", $page => $IkiWiki::DEPEND_LINKS, "" => 1)
 				if match_glob($p, $link, %params);
-			$p=~s/^\///;
+			my ($p_rel)=$p=~/^\/?(.*)/;
 			$link=~s/^\///;
-			return IkiWiki::SuccessReason->new("$page links to page $p matching $link")
-				if match_glob($p, $link, %params);
+			return IkiWiki::SuccessReason->new("$page links to page $p_rel matching $link", $page => $IkiWiki::DEPEND_LINKS, "" => 1)
+				if match_glob($p_rel, $link, %params);
 		}
 	}
-	return IkiWiki::FailReason->new("$page does not link to $link");
+	return IkiWiki::FailReason->new("$page does not link to $link", "" => 1);
 }
 
 sub match_backlink ($$;@) {
-	return match_link($_[1], $_[0], @_);
+	my $ret=match_link($_[1], $_[0], @_);
+	$ret->influences($_[1] => $IkiWiki::DEPEND_LINKS);
+	return $ret;
 }
 
 sub match_created_before ($$;@) {
@@ -2074,14 +2226,14 @@ sub match_created_before ($$;@) {
 
 	if (exists $IkiWiki::pagectime{$testpage}) {
 		if ($IkiWiki::pagectime{$page} < $IkiWiki::pagectime{$testpage}) {
-			return IkiWiki::SuccessReason->new("$page created before $testpage");
+			return IkiWiki::SuccessReason->new("$page created before $testpage", $testpage => $IkiWiki::DEPEND_PRESENCE);
 		}
 		else {
-			return IkiWiki::FailReason->new("$page not created before $testpage");
+			return IkiWiki::FailReason->new("$page not created before $testpage", $testpage => $IkiWiki::DEPEND_PRESENCE);
 		}
 	}
 	else {
-		return IkiWiki::ErrorReason->new("$testpage does not exist");
+		return IkiWiki::ErrorReason->new("$testpage does not exist", $testpage => $IkiWiki::DEPEND_PRESENCE);
 	}
 }
 
@@ -2094,14 +2246,14 @@ sub match_created_after ($$;@) {
 
 	if (exists $IkiWiki::pagectime{$testpage}) {
 		if ($IkiWiki::pagectime{$page} > $IkiWiki::pagectime{$testpage}) {
-			return IkiWiki::SuccessReason->new("$page created after $testpage");
+			return IkiWiki::SuccessReason->new("$page created after $testpage", $testpage => $IkiWiki::DEPEND_PRESENCE);
 		}
 		else {
-			return IkiWiki::FailReason->new("$page not created after $testpage");
+			return IkiWiki::FailReason->new("$page not created after $testpage", $testpage => $IkiWiki::DEPEND_PRESENCE);
 		}
 	}
 	else {
-		return IkiWiki::ErrorReason->new("$testpage does not exist");
+		return IkiWiki::ErrorReason->new("$testpage does not exist", $testpage => $IkiWiki::DEPEND_PRESENCE);
 	}
 }
 
@@ -2137,11 +2289,13 @@ sub match_user ($$;@) {
 	my $user=shift;
 	my %params=@_;
 	
+	my $regexp=IkiWiki::glob2re($user);
+	
 	if (! exists $params{user}) {
 		return IkiWiki::ErrorReason->new("no user specified");
 	}
 
-	if (defined $params{user} && lc $params{user} eq lc $user) {
+	if (defined $params{user} && $params{user}=~/^$regexp$/i) {
 		return IkiWiki::SuccessReason->new("user is $user");
 	}
 	elsif (! defined $params{user}) {
