@@ -30,7 +30,7 @@ sub preprocess_if (@) {
 	}
 
 	my $result=0;
-	if ((exists $params{all} && ! IkiWiki::yesno($params{all})) ||
+	if (! IkiWiki::yesno($params{all}) ||
 	    # An optimisation to avoid needless looping over every page
 	    # for simple uses of some of the tests.
 	    $params{test} =~ /^([\s\!()]*((enabled|sourcepage|destpage|included)\([^)]*\)|(and|or))[\s\!()]*)+$/) {
@@ -59,7 +59,8 @@ sub preprocess_if (@) {
 	else {
 		$ret="";
 	}
-	return IkiWiki::preprocess($params{page}, $params{destpage}, $ret);
+	return IkiWiki::preprocess($params{page}, $params{destpage}, 
+		IkiWiki::filter($params{page}, $params{destpage}, $ret));
 }
 
 package IkiWiki::PageSpec;
