@@ -64,7 +64,7 @@ sub cgi_editpage ($$) {
 
 	decode_cgi_utf8($q);
 
-	my @fields=qw(do rcsinfo subpage from page type editcontent editmessage);
+	my @fields=qw(do rcsinfo subpage from page type editcontent comments);
 	my @buttons=("Save Page", "Preview", "Cancel");
 	eval q{use CGI::FormBuilder};
 	error($@) if $@;
@@ -78,7 +78,7 @@ sub cgi_editpage ($$) {
 		action => $config{cgiurl},
 		header => 0,
 		table => 0,
-		template => { template("editpage.tmpl") },
+		template => scalar template_params("editpage.tmpl"),
 	);
 	
 	decode_form_utf8($form);
@@ -92,9 +92,9 @@ sub cgi_editpage ($$) {
 	# wiki_file_regexp.
 	my ($page)=$form->field('page')=~/$config{wiki_file_regexp}/;
 	$page=possibly_foolish_untaint($page);
-	my $absolute=($page =~ s#^/+##); # absolute name used to force location
+	my $absolute=($page =~ s#^/+##);
 	if (! defined $page || ! length $page ||
-	    file_pruned($page)) {
+	    file_pruned($page, $config{srcdir})) {
 		error(gettext("bad page name"));
 	}
 
@@ -144,16 +144,16 @@ sub cgi_editpage ($$) {
 	$form->field(name => "subpage", type => 'hidden');
 	$form->field(name => "page", value => $page, force => 1);
 	$form->field(name => "type", value => $type, force => 1);
-	$form->field(name => "editmessage", type => "text", size => 80);
+	$form->field(name => "comments", type => "text", size => 80);
 	$form->field(name => "editcontent", type => "textarea", rows => 20,
 		cols => 80);
 	$form->tmpl_param("can_commit", $config{rcs});
+	$form->tmpl_param("indexlink", indexlink());
 	$form->tmpl_param("helponformattinglink",
 		htmllink($page, $page, "ikiwiki/formatting",
 			noimageinline => 1,
 			linktext => "FormattingHelp"));
 	
-	my $previewing=0;
 	if ($form->submitted eq "Cancel") {
 		if ($form->field("do") eq "create" && defined $from) {
 			redirect($q, urlto($from, undef, 1));
@@ -167,11 +167,11 @@ sub cgi_editpage ($$) {
 		exit;
 	}
 	elsif ($form->submitted eq "Preview") {
-		$previewing=1;
-
 		my $new=not exists $pagesources{$page};
-		# temporarily record its type
-		$pagesources{$page}=$page.".".$type if $new;
+		if ($new) {
+			# temporarily record its type
+			$pagesources{$page}=$page.".".$type;
+		}
 		my %wasrendered=map { $_ => 1 } @{$renderedfiles{$page}};
 
 		my $content=$form->field('editcontent');
@@ -196,17 +196,18 @@ sub cgi_editpage ($$) {
 		});
 		$form->tmpl_param("page_preview", $preview);
 		
+		if ($new) {
+			delete $pagesources{$page};
+		}
+
 		# Previewing may have created files on disk.
 		# Keep a list of these to be deleted later.
 		my %previews = map { $_ => 1 } @{$wikistate{editpage}{previews}};
 		foreach my $f (@{$renderedfiles{$page}}) {
 			$previews{$f}=1 unless $wasrendered{$f};
 		}
-
-		# Throw out any other state changes made during previewing,
-		# and save the previews list.
-		loadindex();
 		@{$wikistate{editpage}{previews}} = keys %previews;
+		$renderedfiles{$page}=[keys %wasrendered];
 		saveindex();
 	}
 	elsif ($form->submitted eq "Save Page") {
@@ -219,7 +220,8 @@ sub cgi_editpage ($$) {
 			my $best_loc;
 			if (! defined $from || ! length $from ||
 			    $from ne $form->field('from') ||
-			    file_pruned($from) ||
+			    file_pruned($from, $config{srcdir}) ||
+			    $from=~/^\// || 
 			    $absolute ||
 			    $form->submitted) {
 				@page_locs=$best_loc=$page;
@@ -255,7 +257,7 @@ sub cgi_editpage ($$) {
 			if (! @page_locs) {
 				# hmm, someone else made the page in the
 				# meantime?
-				if ($previewing) {
+				if ($form->submitted eq "Preview") {
 					# let them go ahead with the edit
 					# and resolve the conflict at save
 					# time
@@ -312,8 +314,7 @@ sub cgi_editpage ($$) {
 			$form->title(sprintf(gettext("editing %s"), pagetitle($page)));
 		}
 		
-		showform($form, \@buttons, $session, $q,
-			forcebaseurl => $baseurl, page => $page);
+		showform($form, \@buttons, $session, $q, forcebaseurl => $baseurl);
 	}
 	else {
 		# save page
@@ -330,8 +331,7 @@ sub cgi_editpage ($$) {
 			$form->field(name => "page", type => 'hidden');
 			$form->field(name => "type", type => 'hidden');
 			$form->title(sprintf(gettext("editing %s"), $page));
-			showform($form, \@buttons, $session, $q,
-				forcebaseurl => $baseurl, page => $page);
+			showform($form, \@buttons, $session, $q, forcebaseurl => $baseurl);
 			exit;
 		}
 		elsif ($form->field("do") eq "create" && $exists) {
@@ -345,15 +345,14 @@ sub cgi_editpage ($$) {
 				value => readfile("$config{srcdir}/$file").
 				         "\n\n\n".$form->field("editcontent"),
 				force => 1);
-			showform($form, \@buttons, $session, $q,
-				forcebaseurl => $baseurl, page => $page);
+			showform($form, \@buttons, $session, $q, forcebaseurl => $baseurl);
 			exit;
 		}
 			
 		my $message="";
-		if (defined $form->field('editmessage') &&
-		    length $form->field('editmessage')) {
-			$message=$form->field('editmessage');
+		if (defined $form->field('comments') &&
+		    length $form->field('comments')) {
+			$message=$form->field('comments');
 		}
 		
 		my $content=$form->field('editcontent');
@@ -387,7 +386,7 @@ sub cgi_editpage ($$) {
 			$form->field(name => "type", type => 'hidden');
 			$form->title(sprintf(gettext("editing %s"), $page));
 			showform($form, \@buttons, $session, $q,
-				forcebaseurl => $baseurl, page => $page);
+				forcebaseurl => $baseurl);
 			exit;
 		}
 		
@@ -401,12 +400,9 @@ sub cgi_editpage ($$) {
 			# signaling to it that it should not try to
 			# do anything.
 			disable_commit_hook();
-			$conflict=rcs_commit(
-				file => $file,
-				message => $message,
-				token => $form->field("rcsinfo"),
-				session => $session,
-			);
+			$conflict=rcs_commit($file, $message,
+				$form->field("rcsinfo"),
+				$session->param("name"), $ENV{REMOTE_ADDR});
 			enable_commit_hook();
 			rcs_update();
 		}
@@ -429,7 +425,7 @@ sub cgi_editpage ($$) {
 			$form->field(name => "type", type => 'hidden');
 			$form->title(sprintf(gettext("editing %s"), $page));
 			showform($form, \@buttons, $session, $q,
-				forcebaseurl => $baseurl, page => $page);
+				forcebaseurl => $baseurl);
 		}
 		else {
 			# The trailing question mark tries to avoid broken
