@@ -49,6 +49,7 @@ sub getsetup () {
 		plugin => {
 			safe => 1,
 			rebuild => undef,
+			section => "core",
 		},
 		rss => {
 			type => "boolean",
@@ -159,7 +160,7 @@ sub preprocess_inline (@) {
 	my $rss=(($config{rss} || $config{allowrss}) && exists $params{rss}) ? yesno($params{rss}) : $config{rss};
 	my $atom=(($config{atom} || $config{allowatom}) && exists $params{atom}) ? yesno($params{atom}) : $config{atom};
 	my $quick=exists $params{quick} ? yesno($params{quick}) : 0;
-	my $feeds=exists $params{feeds} ? yesno($params{feeds}) : !$quick;
+	my $feeds=! $nested && (exists $params{feeds} ? yesno($params{feeds}) : !$quick);
 	my $emptyfeeds=exists $params{emptyfeeds} ? yesno($params{emptyfeeds}) : 1;
 	my $feedonly=yesno($params{feedonly});
 	if (! exists $params{show} && ! $archive) {
@@ -195,41 +196,38 @@ sub preprocess_inline (@) {
 
 		@list = map { bestlink($params{page}, $_) }
 		        split ' ', $params{pagenames};
+
+		if (yesno($params{reverse})) {
+			@list=reverse(@list);
+		}
+
+		foreach my $p (@list) {
+			add_depends($params{page}, $p, deptype($quick ? "presence" : "content"));
+		}
 	}
 	else {
-		add_depends($params{page}, $params{pages});
+		my $num=0;
+		if ($params{show}) {
+			$num=$params{show};
+		}
+		if ($params{feedshow} && $num < $params{feedshow} && $num > 0) {
+			$num=$params{feedshow};
+		}
+		if ($params{skip} && $num) {
+			$num+=$params{skip};
+		}
 
-		@list = pagespec_match_list(
-			[ grep { $_ ne $params{page} } keys %pagesources ],
-			$params{pages}, location => $params{page});
-
-		if (exists $params{sort} && $params{sort} eq 'title') {
-			@list=sort { pagetitle(basename($a)) cmp pagetitle(basename($b)) } @list;
-		}
-		elsif (exists $params{sort} && $params{sort} eq 'title_natural') {
-			eval q{use Sort::Naturally};
-			if ($@) {
-				error(gettext("Sort::Naturally needed for title_natural sort"));
-			}
-			@list=sort { Sort::Naturally::ncmp(pagetitle(basename($a)), pagetitle(basename($b))) } @list;
-		}
-		elsif (exists $params{sort} && $params{sort} eq 'mtime') {
-			@list=sort { $pagemtime{$b} <=> $pagemtime{$a} } @list;
-		}
-		elsif (! exists $params{sort} || $params{sort} eq 'age') {
-			@list=sort { $pagectime{$b} <=> $pagectime{$a} } @list;
-		}
-		else {
-			error sprintf(gettext("unknown sort type %s"), $params{sort});
-		}
-	}
-
-	if (yesno($params{reverse})) {
-		@list=reverse(@list);
+		@list = pagespec_match_list($params{page}, $params{pages},
+			deptype => deptype($quick ? "presence" : "content"),
+			filter => sub { $_[0] eq $params{page} },
+			sort => exists $params{sort} ? $params{sort} : "age",
+			reverse => yesno($params{reverse}),
+			($num ? (num => $num) : ()),
+		);
 	}
 
 	if (exists $params{skip}) {
-		@list=@list[$params{skip} .. scalar @list - 1];
+		@list=@list[$params{skip} .. $#list];
 	}
 	
 	my @feedlist;
@@ -247,15 +245,12 @@ sub preprocess_inline (@) {
 		@list=@list[0..$params{show} - 1];
 	}
 
-	# Explicitly add all currently displayed pages as dependencies, so
-	# that if they are removed or otherwise changed, the inline will be
-	# sure to be updated.
-	foreach my $p ($#list >= $#feedlist ? @list : @feedlist) {
-		add_depends($params{page}, $p);
-	}
-	
 	if ($feeds && exists $params{feedpages}) {
-		@feedlist=pagespec_match_list(\@feedlist, $params{feedpages}, location => $params{page});
+		@feedlist = pagespec_match_list(
+			$params{page}, "($params{pages}) and ($params{feedpages})",
+			deptype => deptype($quick ? "presence" : "content"),
+			list => \@feedlist,
+		);
 	}
 
 	my ($feedbase, $feednum);
@@ -338,12 +333,13 @@ sub preprocess_inline (@) {
 			error sprintf(gettext("nonexistant template %s"), $params{template});
 		}
 		my $template=HTML::Template->new(@params) unless $raw;
+		my $needcontent=$raw || (!($archive && $quick) && $template->query(name => 'content'));
 	
 		foreach my $page (@list) {
 			my $file = $pagesources{$page};
 			my $type = pagetype($file);
-			if (! $raw || ($raw && ! defined $type)) {
-				unless ($archive && $quick) {
+			if (! $raw) {
+				if ($needcontent) {
 					# Get the content before populating the
 					# template, since getting the content uses
 					# the same template if inlines are nested.
@@ -362,9 +358,9 @@ sub preprocess_inline (@) {
 					my $file = $pagesources{$page};
 					my $type = pagetype($file);
 					if ($config{discussion}) {
-						if ($page !~ /.*\/\Q$config{discussionpage}\E$/ &&
+						if ($page !~ /.*\/\Q$config{discussionpage}\E$/i &&
 						    (length $config{cgiurl} ||
-						     exists $links{$page."/".$config{discussionpage}})) {
+						     exists $pagesources{$page."/".lc($config{discussionpage})})) {
 							$template->param(have_actions => 1);
 							$template->param(discussionlink =>
 								htmllink($page,
@@ -374,9 +370,12 @@ sub preprocess_inline (@) {
 									forcesubpage => 1));
 						}
 					}
-					if (length $config{cgiurl} && defined $type) {
+					if (length $config{cgiurl} &&
+					    defined $type &&
+					    IkiWiki->can("cgi_editpage")) {
 						$template->param(have_actions => 1);
 						$template->param(editurl => cgiurl(do => "edit", page => $page));
+
 					}
 				}
 	
@@ -396,6 +395,10 @@ sub preprocess_inline (@) {
 					      filter($page, $params{destpage},
 					      readfile(srcfile($file)))));
 				}
+				else {
+					$ret.="\n".
+					      readfile(srcfile($file));
+				}
 			}
 		}
 	}
@@ -424,6 +427,8 @@ sub preprocess_inline (@) {
 		}
 	}
 	
+	clear_inline_content_cache();
+
 	return $ret if $raw || $nested;
 	push @inline, $ret;
 	return "<div class=\"inline\" id=\"$#inline\"></div>\n\n";
@@ -438,25 +443,42 @@ sub pagetemplate_inline (@) {
 		if exists $feedlinks{$page} && $template->query(name => "feedlinks");
 }
 
+{
+my %inline_content;
+my $cached_destpage="";
+
 sub get_inline_content ($$) {
 	my $page=shift;
 	my $destpage=shift;
 	
+	if (exists $inline_content{$page} && $cached_destpage eq $destpage) {
+		return $inline_content{$page};
+	}
+
 	my $file=$pagesources{$page};
 	my $type=pagetype($file);
+	my $ret="";
 	if (defined $type) {
 		$nested++;
-		my $ret=htmlize($page, $destpage, $type,
+		$ret=htmlize($page, $destpage, $type,
 		       linkify($page, $destpage,
 		       preprocess($page, $destpage,
 		       filter($page, $destpage,
 		       readfile(srcfile($file))))));
 		$nested--;
-		return $ret;
 	}
-	else {
-		return "";
+	
+	if ($cached_destpage ne $destpage) {
+		clear_inline_content_cache();
+		$cached_destpage=$destpage;
 	}
+	return $inline_content{$page}=$ret;
+}
+
+sub clear_inline_content_cache () {
+	%inline_content=();
+}
+
 }
 
 sub date_822 ($) {
