@@ -68,6 +68,7 @@ sub getsetup () {
 		plugin => {
 			safe => 0, # rcs plugin
 			rebuild => undef,
+			section => "rcs",
 		},
 		mtn_wrapper => {
 			type => "string",
@@ -228,7 +229,7 @@ sub read_certs ($$) {
 	my @ret;
 
 	my $line = $results[0];
-	while ($line =~ m/\s+key\s"(.*?)"\nsignature\s"(ok|bad|unknown)"\n\s+name\s"(.*?)"\n\s+value\s"(.*?)"\n\s+trust\s"(trusted|untrusted)"\n/sg) {
+	while ($line =~ m/\s+key\s["\[](.*?)[\]"]\nsignature\s"(ok|bad|unknown)"\n\s+name\s"(.*?)"\n\s+value\s"(.*?)"\n\s+trust\s"(trusted|untrusted)"\n/sg) {
 		push @ret, {
 			key => $1,
 			signature => $2,
@@ -558,7 +559,8 @@ sub rcs_recentchanges ($) {
 					# from the changelog
 					if ($cert->{key} eq $config{mtnkey}) {
 						$committype = "web";
-					} else {
+					}
+					else {
 						$committype = "mtn";
 					}
 				} elsif ($cert->{name} eq "date") {
