@@ -96,72 +96,6 @@ sub setpassword ($$;$) {
 	else {
 		IkiWiki::userinfo_set($user, $field, $password);
 	}
-
-	# Setting the password clears any passwordless login token.
-	if ($field ne 'passwordless') {
-		IkiWiki::userinfo_set($user, "passwordless", "");
-	}
-}
-
-# Generates a token that can be used to log the user in.
-# This needs to be hard to guess. Generating a cgi session id will
-# make it as hard to guess as any cgi session.
-sub gentoken ($$;$) {
-	my $user=shift;
-	my $tokenfield=shift;
-	my $reversable=shift;
-
-	eval q{use CGI::Session};
-	error($@) if $@;
-	my $token = CGI::Session->new->id;
-	if (! $reversable) {
-		setpassword($user, $token, $tokenfield);
-	}
-	else {
-		IkiWiki::userinfo_set($user, $tokenfield, $token);
-	}
-	return $token;
-}
-
-# An anonymous user has no normal password, only a passwordless login
-# token. Given an email address, this sets up such a user for that email,
-# unless one already exists, and returns the username.
-sub anonuser ($) {
-	my $email=shift;
-
-	# Want a username for this email that won't overlap with any other.
-	my $user=$email;
-	$user=~s/@/_/g;
-
-	my $userinfo=IkiWiki::userinfo_retrieve();
-	if (! exists $userinfo->{$user} || ! ref $userinfo->{$user}) {
-		if (IkiWiki::userinfo_setall($user, {
-		    	'email' => $email,
-			'regdate' => time})) {
-			gentoken($user, "passwordless", 1);
-			return $user;
-		}
-		else {
-			error(gettext("Error creating account."));
-		}
-	}
-	elsif (defined anonusertoken($userinfo->{$user})) {
-		return $user;
-	}
-	else {
-		return undef;
-	}
-}
-
-sub anonusertoken ($) {
-	my $userhash=shift;
-	if (exists $userhash->{passwordless} &&
-	    length $userhash->{passwordless}) {
-		return $userhash->{passwordless};
-	}
-	else {
-		return undef;
-	}
 }
 
 sub formbuilder_setup (@) {
@@ -171,13 +105,11 @@ sub formbuilder_setup (@) {
 	my $session=$params{session};
 	my $cgi=$params{cgi};
 
-	my $do_register=defined $cgi->param("do") && $cgi->param("do") eq "register";
-
-	if ($form->title eq "signin" || $form->title eq "register" || $do_register) {
+	if ($form->title eq "signin" || $form->title eq "register" || $cgi->param("do") eq "register") {
 		$form->field(name => "name", required => 0);
 		$form->field(name => "password", type => "password", required => 0);
 		
-		if ($form->submitted eq "Register" || $form->submitted eq "Create Account" || $do_register) {
+		if ($form->submitted eq "Register" || $form->submitted eq "Create Account" || $cgi->param("do") eq "register") {
 			$form->field(name => "confirm_password", type => "password");
 			$form->field(name => "account_creation_password", type => "password")
 				 if (defined $config{account_creation_password} &&
@@ -315,10 +247,8 @@ sub formbuilder (@) {
 	my $cgi=$params{cgi};
 	my $buttons=$params{buttons};
 
-	my $do_register=defined $cgi->param("do") && $cgi->param("do") eq "register";
-
 	if ($form->title eq "signin" || $form->title eq "register") {
-		if (($form->submitted && $form->validate) || $do_register) {
+		if (($form->submitted && $form->validate) || $cgi->param("do") eq "register") {
 			if ($form->submitted eq 'Login') {
 				$session->param("name", $form->field("name"));
 				IkiWiki::cgi_postsignin($cgi, $session);
@@ -343,20 +273,27 @@ sub formbuilder (@) {
 				if (! length $email) {
 					error(gettext("No email address, so cannot email password reset instructions."));
 				}
-
-				my $token=gentoken($user_name, "resettoken");
+				
+				# Store a token that can be used once
+				# to log the user in. This needs to be hard
+				# to guess. Generating a cgi session id will
+				# make it as hard to guess as any cgi session.
+				eval q{use CGI::Session};
+				error($@) if $@;
+				my $token = CGI::Session->new->id;
+				setpassword($user_name, $token, "resettoken");
 				
 				my $template=template("passwordmail.tmpl");
 				$template->param(
 					user_name => $user_name,
-					passwordurl => IkiWiki::cgiurl_abs(
+					passwordurl => IkiWiki::cgiurl(
 						'do' => "reset",
 						'name' => $user_name,
 						'token' => $token,
 					),
 					wikiurl => $config{url},
 					wikiname => $config{wikiname},
-					remote_addr => $session->remote_addr(),
+					REMOTE_ADDR => $ENV{REMOTE_ADDR},
 				);
 				
 				eval q{use Mail::Sendmail};
@@ -374,7 +311,7 @@ sub formbuilder (@) {
 				$form->field(name => "name", required => 0);
 				push @$buttons, "Reset Password";
 			}
-			elsif ($form->submitted eq "Register" || $do_register) {
+			elsif ($form->submitted eq "Register" || $cgi->param("do") eq "register") {
 				@$buttons="Create Account";
 			}
 		}
@@ -388,7 +325,7 @@ sub formbuilder (@) {
 	elsif ($form->title eq "preferences") {
 		if ($form->submitted eq "Save Preferences" && $form->validate) {
 			my $user_name=$form->field('name');
-			if (defined $form->field("password") && length $form->field("password")) {
+			if ($form->field("password") && length $form->field("password")) {
 				setpassword($user_name, $form->field('password'));
 			}
 		}
@@ -415,22 +352,6 @@ sub sessioncgi ($$) {
 		IkiWiki::cgi_prefs($q, $session);
 		exit;
 	}
-	elsif ($q->param('do') eq 'tokenauth') {
-		my $name=$q->param("name");
-		my $token=$q->param("token");
-
-		if (! defined $name || ! defined $token ||
-		    ! length $name  || ! length $token) {
-			error(gettext("incorrect url"));
-	 	}
-		if (! checkpassword($name, $token, "passwordless")) {
-			error(gettext("access denied"));
-		}
-
-		$session->param("name", $name);
-		IkiWiki::cgi_prefs($q, $session);
-		exit;
-	}
 	elsif ($q->param("do") eq "register") {
 		# After registration, need to go somewhere, so show prefs page.
 		$session->param(postsignin => "do=prefs");
