@@ -3,25 +3,31 @@ package IkiWiki::Plugin::poll;
 
 use warnings;
 use strict;
-use IkiWiki 2.00;
+use IkiWiki 3.00;
+use Encode;
 
-sub import { #{{{
+sub import {
+	hook(type => "getsetup", id => "poll", call => \&getsetup);
 	hook(type => "preprocess", id => "poll", call => \&preprocess);
-	hook(type => "cgi", id => "poll", call => \&cgi);
-} # }}}
+	hook(type => "sessioncgi", id => "poll", call => \&sessioncgi);
+}
 
-sub yesno ($) { #{{{
-	my $val=shift;
-	return (defined $val && lc($val) eq "yes");
-} #}}}
+sub getsetup () {
+	return 
+		plugin => {
+			safe => 1,
+			rebuild => undef,
+			section => "widget",
+		},
+}
 
 my %pagenum;
-sub preprocess (@) { #{{{
+sub preprocess (@) {
 	my %params=(open => "yes", total => "yes", percent => "yes", @_);
 
-	my $open=yesno($params{open});
-	my $showtotal=yesno($params{total});
-	my $showpercent=yesno($params{percent});
+	my $open=IkiWiki::yesno($params{open});
+	my $showtotal=IkiWiki::yesno($params{total});
+	my $showpercent=IkiWiki::yesno($params{percent});
 	$pagenum{$params{page}}++;
 
 	my %choices;
@@ -72,12 +78,13 @@ sub preprocess (@) { #{{{
 		$ret.="<span>".gettext("Total votes:")." $total</span>\n";
 	}
 	return "<div class=poll>$ret</div>";
-} # }}}
+}
 
-sub cgi ($) { #{{{
+sub sessioncgi ($$) {
 	my $cgi=shift;
+	my $session=shift;
 	if (defined $cgi->param('do') && $cgi->param('do') eq "poll") {
-		my $choice=$cgi->param('choice');
+		my $choice=decode_utf8($cgi->param('choice'));
 		if (! defined $choice) {
 			error("no choice specified");
 		}
@@ -92,15 +99,16 @@ sub cgi ($) { #{{{
 
 		# Did they vote before? If so, let them change their vote,
 		# and check for dups.
-		my $session=IkiWiki::cgi_getsession();
 		my $choice_param="poll_choice_${page}_$num";
 		my $oldchoice=$session->param($choice_param);
 		if (defined $oldchoice && $oldchoice eq $choice) {
 			# Same vote; no-op.
-			IkiWiki::redirect($cgi, "$config{url}/".htmlpage($page));
+			IkiWiki::redirect($cgi, urlto($page, undef, 1));
 			exit;
 		}
 
+		my $prefix=$config{prefix_directives} ? "!poll" : "poll";
+
 		my $content=readfile(srcfile($pagesources{$page}));
 		# Now parse the content, find the right poll,
 		# and find the choice within it, and increment its number.
@@ -108,16 +116,16 @@ sub cgi ($) { #{{{
 		my $edit=sub {
 			my $escape=shift;
 			my $params=shift;
-			return "\\[[poll $params]]" if $escape;
+			return "\\[[$prefix $params]]" if $escape;
 			if (--$num == 0) {
 				$params=~s/(^|\s+)(\d+)\s+"?\Q$choice\E"?(\s+|$)/$1.($2+1)." \"$choice\"".$3/se;
 				if (defined $oldchoice) {
 					$params=~s/(^|\s+)(\d+)\s+"?\Q$oldchoice\E"?(\s+|$)/$1.($2-1 >=0 ? $2-1 : 0)." \"$oldchoice\"".$3/se;
 				}
 			}
-			return "[[poll $params]]";
+			return "[[$prefix $params]]";
 		};
-		$content =~ s{(\\?)\[\[poll\s+([^]]+)\s*\]\]}{$edit->($1, $2)}seg;
+		$content =~ s{(\\?)\[\[\Q$prefix\E\s+([^]]+)\s*\]\]}{$edit->($1, $2)}seg;
 
 		# Store their vote, update the page, and redirect to it.
 		writefile($pagesources{$page}, $config{srcdir}, $content);
@@ -142,9 +150,9 @@ sub cgi ($) { #{{{
 		error($@) if $@;
 		my $cookie = CGI::Cookie->new(-name=> $session->name, -value=> $session->id);
 		print $cgi->redirect(-cookie => $cookie,
-			-url => "$config{url}/".htmlpage($page));
+			-url => urlto($page, undef, 1));
 		exit;
 	}
-} #}}}
+}
 
 1
