@@ -50,7 +50,8 @@ sub preprocess_toggleable (@) {
 
 	# Preprocess the text to expand any preprocessor directives
 	# embedded inside it.
-	$params{text}=IkiWiki::preprocess($params{page}, $params{destpage}, $params{text});
+	$params{text}=IkiWiki::preprocess($params{page}, $params{destpage}, 
+		IkiWiki::filter($params{page}, $params{destpage}, $params{text}));
 	
 	my $id=genid($params{page}, $params{id});
 	my $class=(lc($params{open}) ne "yes") ? "toggleable" : "toggleable-open";
@@ -80,9 +81,9 @@ sub include_javascript ($;$) {
 	my $page=shift;
 	my $absolute=shift;
 	
-	return '<script src="'.urlto("ikiwiki/ikiwiki.js", $page, $absolute).
+	return '<script src="'.urlto("ikiwiki.js", $page, $absolute).
 		'" type="text/javascript" charset="utf-8"></script>'."\n".
-		'<script src="'.urlto("ikiwiki/toggle.js", $page, $absolute).
+		'<script src="'.urlto("toggle.js", $page, $absolute).
 		'" type="text/javascript" charset="utf-8"></script>';
 }
 
