@@ -4,6 +4,8 @@ package IkiWiki::Plugin::wmd;
 use warnings;
 use strict;
 use IkiWiki 3.00;
+use POSIX;
+use Encode;
 
 sub import {
 	add_underlay("wmd");
